#include <timer.h>
#include <stdlib.h>
#include "xmodem.h"
#include <serial.h>

/* purge data at serial port */
static void purge(void)
{
	timer_t timer;

	timer_setup(&timer, 1000);
	while (1) {

		if (serial_tstc()) {
			serial_getc();
			timer_setup(&timer, 1000);
		}

		if (timer_expired(&timer))
			return;
	}
}

/* Send NAK signal to serial port */
uint8_t send_nak(void)
{
	purge();
	serial_putc(NAK);
	retry_count++;
	if (retry_count == 10)
		return 0;
	return 1;
}


/* Send ACK signal to serial port */
void send_ack(void)
{
	serial_putc(ACK);
}

/* Read single byte from serial port */
uint8_t read_byte(void)
{
	return serial_getc();
}

/* Check packet number for received packet */
uint8_t check_packet_number(void)
{
	uint8_t seq1;
	uint8_t seq2;

	/* Get packet number */
	seq1 = serial_getc() & 0xFF;

	/* Get inverses packet number */
	seq2 = (~serial_getc()) & 0xFF;

	if (seq1 == seq2) {
		seq++;
		return 1;
	} else {
		return 0;
	}
}

/* Calculate Check-sum for received packet */
uint8_t checksum(uint8_t buff[])
{
	uint32_t i;
	uint32_t sum;
	uint8_t data;

	/* Get 128 bytes Data from software FIFO buffer
	 * and Calculate check-sum value */
	for (sum = 0, i = 0; i < 128; i++) {
		data = serial_getc();
		buff[i] = data;
		sum += data;
	}

	/* Get Check-sum byte from software FIFO buffer */
	data = serial_getc();

	/* Compare Check-sum value and Check-sum byte */
	if ((sum & 0xff) == (data & 0xff)) {
		csum++;
		return 1;
	} else {
		return 0;
	}
}

/*
 * Return 128 bytes data from received packet
 * whether if it is receive correct packet
 * else it will retry to get correct packet or End of transmission.
 */
uint8_t *xreceive(uint8_t buff[])
{
	uint8_t head;
	static uint8_t handle;

	if (handle) {
		/* After first packet of transmission */
		send_ack();
	} else {
		/* Beginning of transmission */
		send_nak();
	}

	while (1) {
		/* Read packet header */
		head = read_byte();

		switch (head) {
		case EOT:
			send_ack();
			return NULL;
		case SOH:
			if (check_packet_number()) {
				if (checksum(buff)) {
					retry_count = 0;
					handle = 1;
					return buff;
				} else {
					if (send_nak())
						return NULL;
				}
			} else {
				if (send_nak())
					return NULL;
			}
			break;
		default:
			if (send_nak())
				return NULL;
			break;
		}
	}
}
